@@ -1,6 +1,6 @@
 /* hexter DSSI software synthesizer plugin
  *
- * Copyright (C) 2004, 2009 Sean Bolton and others.
+ * Copyright (C) 2004, 2009, 2011 Sean Bolton and others.
  *
  * Portions of this file may have come from Juan Linietsky's
  * rx-saturno, copyright (C) 2002 by Juan Linietsky.
@@ -29,22 +29,26 @@
  * Boston, MA 02110-1301 USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #define _BSD_SOURCE    1
 #define _SVID_SOURCE   1
 #define _ISOC99_SOURCE 1
 
 #include <math.h>
 
+#include "hexter_types.h"
 #include "dx7_voice.h"
 
 static int dx7_voice_tables_initialized = 0;
-int32_t    dx7_voice_sin_table[SINE_SIZE + 1];
 
-extern int32_t dx7_voice_eg_ol_to_mod_index_table[257]; /* forward */
-extern int32_t dx7_voice_eg_ol_to_amp_table[257];       /* forward */
+dx7_sample_t    dx7_voice_sin_table[SINE_SIZE + 1];
 
-int32_t       *dx7_voice_eg_ol_to_mod_index = &dx7_voice_eg_ol_to_mod_index_table[128];
-int32_t       *dx7_voice_eg_ol_to_amp       = &dx7_voice_eg_ol_to_amp_table[128];
+extern dx7_sample_t dx7_voice_eg_ol_to_mod_index_table[257]; /* forward */
+
+dx7_sample_t  *dx7_voice_eg_ol_to_mod_index = &dx7_voice_eg_ol_to_mod_index_table[128];
 
 void dx7_voice_init_tables(void) {
 
@@ -61,14 +65,15 @@ void dx7_voice_init_tables(void) {
             dx7_voice_sin_table[i] = DOUBLE_TO_FP(f);
         }
 
+#ifndef HEXTER_USE_FLOATING_POINT
 #if FP_SHIFT != 24
-        /* The fixed-point tables below are all in s7.24 format.  Shift
+        /* Any fixed-point tables below are in s7.24 format.  Shift
          * them to match FP_SHIFT. */
         for (i = 0; i <= 256; i++) {
             dx7_voice_eg_ol_to_mod_index_table[i] >>= (24 - FP_SHIFT);
-            dx7_voice_eg_ol_to_amp_table[i]       >>= (24 - FP_SHIFT);
         }
 #endif
+#endif /* ! HEXTER_USE_FLOATING_POINT */
 
         dx7_voice_tables_initialized = 1;
     }
@@ -119,134 +124,10 @@ float dx7_voice_carrier_count[32] = {
     5.0f, 3.0f, 3.0f, 3.0f, 4.0f, 4.0f, 5.0f, 6.0f
 };
 
-float dx7_voice_eg_rate_rise_duration[128] = {  /* generated from my f04new */
-
-     39.638000,  37.013000,  34.388000,  31.763000,  27.210500,
-     22.658000,  20.408000,  18.158000,  15.908000,  14.557000,
-     13.206000,  12.108333,  11.010667,   9.913000,   8.921000,
-      7.929000,   7.171333,   6.413667,   5.656000,   5.307000,
-      4.958000,   4.405667,   3.853333,   3.301000,   2.889000,
-      2.477000,   2.313000,   2.149000,   1.985000,   1.700500,
-      1.416000,   1.274333,   1.132667,   0.991000,   0.909000,
-      0.827000,   0.758000,   0.689000,   0.620000,   0.558000,
-      0.496000,   0.448667,   0.401333,   0.354000,   0.332000,
-      0.310000,   0.275667,   0.241333,   0.207000,   0.180950,
-      0.154900,   0.144567,   0.134233,   0.123900,   0.106200,
-      0.088500,   0.079667,   0.070833,   0.062000,   0.056800,
-      0.051600,   0.047300,   0.043000,   0.038700,   0.034800,
-      0.030900,   0.028000,   0.025100,   0.022200,   0.020815,
-      0.019430,   0.017237,   0.015043,   0.012850,   0.011230,
-      0.009610,   0.009077,   0.008543,   0.008010,   0.006960,
-      0.005910,   0.005357,   0.004803,   0.004250,   0.003960,
-      0.003670,   0.003310,   0.002950,   0.002590,   0.002420,
-      0.002250,   0.002000,   0.001749,   0.001499,   0.001443,
-      0.001387,   0.001242,   0.001096,   0.000951,   0.000815,
-      0.000815,   0.000815,   0.000815,   0.000815,   0.000815,
-      0.000815,   0.000815,   0.000815,   0.000815,   0.000815,
-      0.000815,   0.000815,   0.000815,   0.000815,   0.000815,
-      0.000815,   0.000815,   0.000815,   0.000815,   0.000815,
-      0.000815,   0.000815,   0.000815,   0.000815,   0.000815,
-      0.000815,   0.000815,   0.000815
-
-};
-
-float dx7_voice_eg_rate_decay_duration[128] = {  /* generated from my f06new */
-
-    317.487000, 285.764500, 254.042000, 229.857000, 205.672000,
-    181.487000, 170.154000, 158.821000, 141.150667, 123.480333,
-    105.810000,  98.382500,  90.955000,  81.804667,  72.654333,
-     63.504000,  58.217000,  52.930000,  48.512333,  44.094667,
-     39.677000,  33.089000,  26.501000,  24.283333,  22.065667,
-     19.848000,  17.881500,  15.915000,  14.389667,  12.864333,
-     11.339000,  10.641000,   9.943000,   8.833333,   7.723667,
-      6.614000,   6.149500,   5.685000,   5.112667,   4.540333,
-      3.968000,   3.639000,   3.310000,   3.033667,   2.757333,
-      2.481000,   2.069500,   1.658000,   1.518667,   1.379333,
-      1.240000,   1.116500,   0.993000,   0.898333,   0.803667,
-      0.709000,   0.665500,   0.622000,   0.552667,   0.483333,
-      0.414000,   0.384500,   0.355000,   0.319333,   0.283667,
-      0.248000,   0.228000,   0.208000,   0.190600,   0.173200,
-      0.155800,   0.129900,   0.104000,   0.095400,   0.086800,
-      0.078200,   0.070350,   0.062500,   0.056600,   0.050700,
-      0.044800,   0.042000,   0.039200,   0.034833,   0.030467,
-      0.026100,   0.024250,   0.022400,   0.020147,   0.017893,
-      0.015640,   0.014305,   0.012970,   0.011973,   0.010977,
-      0.009980,   0.008310,   0.006640,   0.006190,   0.005740,
-      0.005740,   0.005740,   0.005740,   0.005740,   0.005740,
-      0.005740,   0.005740,   0.005740,   0.005740,   0.005740,
-      0.005740,   0.005740,   0.005740,   0.005740,   0.005740,
-      0.005740,   0.005740,   0.005740,   0.005740,   0.005740,
-      0.005740,   0.005740,   0.005740,   0.005740,   0.005740,
-      0.005740,   0.005740,   0.005740
-
-};
-
-float dx7_voice_eg_rate_decay_percent[128] = {  /* generated from P/H/Op f07 */
-
-    0.000010, 0.025009, 0.050008, 0.075007, 0.100006,
-    0.125005, 0.150004, 0.175003, 0.200002, 0.225001,
-    0.250000, 0.260000, 0.270000, 0.280000, 0.290000,
-    0.300000, 0.310000, 0.320000, 0.330000, 0.340000,
-    0.350000, 0.358000, 0.366000, 0.374000, 0.382000,
-    0.390000, 0.398000, 0.406000, 0.414000, 0.422000,
-    0.430000, 0.439000, 0.448000, 0.457000, 0.466000,
-    0.475000, 0.484000, 0.493000, 0.502000, 0.511000,
-    0.520000, 0.527000, 0.534000, 0.541000, 0.548000,
-    0.555000, 0.562000, 0.569000, 0.576000, 0.583000,
-    0.590000, 0.601000, 0.612000, 0.623000, 0.634000,
-    0.645000, 0.656000, 0.667000, 0.678000, 0.689000,
-    0.700000, 0.707000, 0.714000, 0.721000, 0.728000,
-    0.735000, 0.742000, 0.749000, 0.756000, 0.763000,
-    0.770000, 0.777000, 0.784000, 0.791000, 0.798000,
-    0.805000, 0.812000, 0.819000, 0.826000, 0.833000,
-    0.840000, 0.848000, 0.856000, 0.864000, 0.872000,
-    0.880000, 0.888000, 0.896000, 0.904000, 0.912000,
-    0.920000, 0.928889, 0.937778, 0.946667, 0.955556,
-    0.964444, 0.973333, 0.982222, 0.991111, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000
-
-};
-
-float dx7_voice_eg_rate_rise_percent[128] = {  /* checked, matches P/H/Op f05 */
-
-    0.000010, 0.000010, 0.000010, 0.000010, 0.000010,
-    0.000010, 0.000010, 0.000010, 0.000010, 0.000010,
-    0.000010, 0.000010, 0.000010, 0.000010, 0.000010,
-    0.000010, 0.000010, 0.000010, 0.000010, 0.000010,
-    0.000010, 0.000010, 0.000010, 0.000010, 0.000010,
-    0.000010, 0.000010, 0.000010, 0.000010, 0.000010,
-    0.000010, 0.000010, 0.005007, 0.010005, 0.015003,
-    0.020000, 0.028000, 0.036000, 0.044000, 0.052000,
-    0.060000, 0.068000, 0.076000, 0.084000, 0.092000,
-    0.100000, 0.108000, 0.116000, 0.124000, 0.132000,
-    0.140000, 0.150000, 0.160000, 0.170000, 0.180000,
-    0.190000, 0.200000, 0.210000, 0.220000, 0.230000,
-    0.240000, 0.251000, 0.262000, 0.273000, 0.284000,
-    0.295000, 0.306000, 0.317000, 0.328000, 0.339000,
-    0.350000, 0.365000, 0.380000, 0.395000, 0.410000,
-    0.425000, 0.440000, 0.455000, 0.470000, 0.485000,
-    0.500000, 0.520000, 0.540000, 0.560000, 0.580000,
-    0.600000, 0.620000, 0.640000, 0.660000, 0.680000,
-    0.700000, 0.732000, 0.764000, 0.796000, 0.828000,
-    0.860000, 0.895000, 0.930000, 0.965000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
-    1.000000, 1.000000, 1.000000
-
-};
-
-/* This table converts an output level of 0 to 99 into a phase modulation
- * index of 0 to ~2.089 periods, expressed in s7.24 fixed point.  It
- * actually extends below 0 and beyond 99, since amplitude modulation can
- * produce 'negative' output levels, and velocities above 100 can produce
+/* This table converts an output level of 0 to 99 into a phase
+ * modulation index of 0 to ~2.089 periods.  It actually extends
+ * below 0 and beyond 99, since amplitude modulation can produce
+ * 'negative' output levels, and velocities above 100 can produce
  * output levels above 99, plus it includes a 257th 'guard' point.
  * Table index 128 corresponds to output level 0, and index 227 to OL 99.
  * I believe this is based on information from the Chowning/Bristow
@@ -277,11 +158,16 @@ float dx7_voice_eg_rate_rise_percent[128] = {  /* checked, matches P/H/Op f05 */
  *        } else {
  *            mi = pow(2.0, ( (33.0/16.0) - ((double)tl_table[ol]/8.0) - 1.0));
  *        }
+ *    #ifndef HEXTER_USE_FLOATING_POINT
  *        printf(" %6d,", DOUBLE_TO_FP(mi));
+ *    #else
+ *        printf(" %g,", mi);
+ *    #endif
  *    }
  */
-int32_t dx7_voice_eg_ol_to_mod_index_table[257] = {
-
+dx7_sample_t dx7_voice_eg_ol_to_mod_index_table[257] = {
+#ifndef HEXTER_USE_FLOATING_POINT
+    /* phase modulation index, expressed in s7.24 fixed point */
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -309,42 +195,57 @@ int32_t dx7_voice_eg_ol_to_mod_index_table[257] = {
     166679334, 181765102, 198216249, 216156353, 235720174,
     257054673, 280320108, 305691246, 333358668, 363530205,
     396432499, 396432499
-
-};
-
-/* This table converts an output level of 0 to 99 into an output amplitude
- * scale factor of 0 to 1 in s7.24 fixed point.  It converts a linear 0-99
- * envelope to an exponential volume, and is just a normalized version of
- * dx7_voice_eg_ol_to_mod_index_table[] (see its description above). */
-int32_t dx7_voice_eg_ol_to_amp_table[257] = {
-
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-    0, 315, 631, 947, 1263, 1579, 2048, 2435, 2896, 3444, 4096,
-    4870, 5792, 6888, 8192, 9741, 10623, 11585, 13777, 15024, 17866,
-    19483, 21247, 23170, 25267, 27554, 30048, 32768, 35733, 38967,
-    42494, 46340, 50535, 55108, 60096, 65536, 71467, 77935, 84989,
-    92681, 101070, 110217, 120193, 131072, 142935, 155871, 169979,
-    185363, 202140, 220435, 240387, 262144, 285870, 311743, 339958,
-    370727, 404281, 440871, 480774, 524288, 571740, 623487, 679917,
-    741455, 808562, 881743, 961548, 1048576, 1143480, 1246974,
-    1359834, 1482910, 1617125, 1763487, 1923096, 2097152, 2286960,
-    2493948, 2719669, 2965820, 3234250, 3526975, 3846193, 4194304,
-    4573920, 4987896, 5439339, 5931641, 6468501, 7053950, 7692387,
-    8388608, 9147841, 9975792, 10878678, 11863283, 12937002,
-    14107900, 15384774, 16777216, 18295683, 19951584, 21757357,
-    23726566, 25874004, 28215801, 30769549, 33554432, 36591367,
-    39903169, 43514714, 47453132, 51748008, 56431603, 61539099,
-    67108864, 73182735, 79806338, 87029429, 94906265, 103496016,
-    112863206, 123078199, 134217728, 146365470, 159612677,
-    174058858, 189812531, 189812531
-
+#else /* HEXTER_USE_FLOATING_POINT */
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f,
+    0.0f, 3.93186e-05f, 7.86372e-05f, 0.000117956f,
+    0.000157274f, 0.000196593f, 0.00025495f, 0.000303188f,
+    0.000360553f, 0.000428773f, 0.000509899f, 0.000606376f,
+    0.000721107f, 0.000857545f, 0.0010198f, 0.00121275f,
+    0.00132252f, 0.00144221f, 0.00171509f, 0.00187032f,
+    0.0022242f, 0.0024255f, 0.00264503f, 0.00288443f,
+    0.00314549f, 0.00343018f, 0.00374064f, 0.00407919f,
+    0.00444839f, 0.00485101f, 0.00529006f, 0.00576885f,
+    0.00629098f, 0.00686036f, 0.00748128f, 0.00815839f,
+    0.00889679f, 0.00970201f, 0.0105801f, 0.0115377f,
+    0.012582f, 0.0137207f, 0.0149626f, 0.0163168f,
+    0.0177936f, 0.019404f, 0.0211602f, 0.0230754f,
+    0.0251639f, 0.0274414f, 0.0299251f, 0.0326336f,
+    0.0355871f, 0.0388081f, 0.0423205f, 0.0461508f,
+    0.0503278f, 0.0548829f, 0.0598502f, 0.0652671f,
+    0.0711743f, 0.0776161f, 0.084641f, 0.0923016f,
+    0.100656f, 0.109766f, 0.1197f, 0.130534f,
+    0.142349f, 0.155232f, 0.169282f, 0.184603f,
+    0.201311f, 0.219532f, 0.239401f, 0.261068f,
+    0.284697f, 0.310464f, 0.338564f, 0.369207f,
+    0.402623f, 0.439063f, 0.478802f, 0.522137f,
+    0.569394f, 0.620929f, 0.677128f, 0.738413f,
+    0.805245f, 0.878126f, 0.957603f, 1.04427f,
+    1.13879f, 1.24186f, 1.35426f, 1.47683f,
+    1.61049f, 1.75625f, 1.91521f, 2.08855f,
+    2.27758f, 2.48372f, 2.70851f, 2.95365f,
+    3.22098f, 3.5125f, 3.83041f, 4.1771f,
+    4.55515f, 4.96743f, 5.41702f, 5.9073f,
+    6.44196f, 7.02501f, 7.66083f, 8.35419f,
+    9.11031f, 9.93486f, 10.834f, 11.8146f,
+    12.8839f, 14.05f, 15.3217f, 16.7084f,
+    18.2206f, 19.8697f, 21.6681f, 23.6292f,
+    23.6292f
+#endif /* HEXTER_USE_FLOATING_POINT */
 };
 
 /* This table lists the output level adjustment needed for a certain
@@ -387,38 +288,6 @@ float dx7_voice_velocity_ol_adjustment[128] = {
 
 };
 
-/* This table converts pitch envelope level parameters into the
- * actual pitch shift in semitones.  For levels [17,85], this is
- * just ((level - 50) / 32 * 12), but at the outer edges the shift
- * is exagerated to 0 = -48 and 99 => 47.624.  This is based on
- * measurements I took from my TX7. */
-double dx7_voice_pitch_level_to_shift[128] = {
-
-    -48.000000, -43.497081, -38.995993, -35.626132, -31.873615,
-    -28.495880, -25.500672, -22.872620, -20.998167, -19.496961,
-    -18.373238, -17.251065, -16.122139, -15.375956, -14.624487,
-    -13.876516, -13.126351, -12.375000, -12.000000, -11.625000,
-    -11.250000, -10.875000, -10.500000, -10.125000, -9.750000,
-    -9.375000, -9.000000, -8.625000, -8.250000, -7.875000,
-    -7.500000, -7.125000, -6.750000, -6.375000, -6.000000,
-    -5.625000, -5.250000, -4.875000, -4.500000, -4.125000,
-    -3.750000, -3.375000, -3.000000, -2.625000, -2.250000,
-    -1.875000, -1.500000, -1.125000, -0.750000, -0.375000, 0.000000,
-    0.375000, 0.750000, 1.125000, 1.500000, 1.875000, 2.250000,
-    2.625000, 3.000000, 3.375000, 3.750000, 4.125000, 4.500000,
-    4.875000, 5.250000, 5.625000, 6.000000, 6.375000, 6.750000,
-    7.125000, 7.500000, 7.875000, 8.250000, 8.625000, 9.000000,
-    9.375000, 9.750000, 10.125000, 10.500000, 10.875000, 11.250000,
-    11.625000, 12.000000, 12.375000, 12.750000, 13.125000,
-    14.251187, 15.001922, 16.126327, 17.250917, 18.375718,
-    19.877643, 21.753528, 24.373913, 27.378021, 30.748956,
-    34.499234, 38.627888, 43.122335, 47.624065, 48.0, 48.0, 48.0,
-    48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0,
-    48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0, 48.0,
-    48.0, 48.0, 48.0, 48.0, 48.0
-
-};
-
 /* This table converts LFO speed to frequency in Hz. It is based on
  * interpolation of Jamie Bullock's measurements. */
 float dx7_voice_lfo_frequency[128] = {
