@@ -1,6 +1,6 @@
 /* hexter DSSI software synthesizer plugin
  *
- * Copyright (C) 2004, 2009 Sean Bolton and others.
+ * Copyright (C) 2004, 2009, 2011 Sean Bolton and others.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,11 +23,12 @@
 
 #include <stdint.h>
 
-#define DX7_VOICE_SIZE_PACKED     128
-#define DX7_VOICE_SIZE_UNPACKED   155
-#define DX7_PERFORMANCE_SIZE       64
-#define DX7_DUMP_SIZE_SINGLE      155+8
-#define DX7_DUMP_SIZE_BULK        4096+8
+#define DX7_VOICE_SIZE_PACKED          128
+#define DX7_VOICE_SIZE_UNPACKED        155
+#define DX7_VOICE_PARAMETERS           146
+#define DX7_PERFORMANCE_SIZE            64
+#define DX7_DUMP_SIZE_VOICE_SINGLE   155+8
+#define DX7_DUMP_SIZE_VOICE_BULK    4096+8
 
 typedef struct _hexter_synth_t    hexter_synth_t;
 typedef struct _hexter_instance_t hexter_instance_t;
@@ -39,6 +40,14 @@ typedef struct _dx7_pitch_eg_t    dx7_pitch_eg_t;
 typedef struct _dx7_portamento_t  dx7_portamento_t;
 typedef struct _dx7_op_t          dx7_op_t;
 
+#ifndef HEXTER_USE_FLOATING_POINT
+// #warning Note: using fixed point
+typedef int32_t dx7_sample_t;
+#else /* HEXTER_USE_FLOATING_POINT */
+// #warning Note: using floating point
+typedef float   dx7_sample_t;
+#endif /* HEXTER_USE_FLOATING_POINT */
+
 /* in hexter.c: */
 int   dssp_voicelist_mutex_lock(void);
 int   dssp_voicelist_mutex_unlock(void);
